@@ -1,66 +1,50 @@
 package Mojolicious::Plugin::Surveil;
 use Mojo::Base 'Mojolicious::Plugin';
 
-use Mojo::JSON qw(encode_json decode_json);
+use Mojo::JSON qw(decode_json encode_json);
 
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 
 sub register {
   my ($self, $app, $config) = @_;
 
-  $config->{events} ||= [qw(click touchstart touchcancel touchend)];
+  $config->{enable_param} ||= '_surveil';
+  $config->{events}       ||= [qw(blur click focus touchstart touchcancel touchend)];
+  $config->{handler}      ||= \&_default_message_handler;
+  $config->{path}         ||= '/mojolicious/plugin/surveil';
 
   push @{$app->renderer->classes}, __PACKAGE__;
-  $self->_after_render_hook($app, $config);
-  $self->_default_route($app, $config) unless $config->{path};
-}
-
-sub _after_render_hook {
-  my ($self, $app, $config) = @_;
-  my $enable_param = $config->{enable_param};
-
-  $app->hook(
-    after_render => sub {
-      my ($c, $output, $format) = @_;
-      return if $format ne 'html';
-      return if $enable_param and !$c->param($enable_param);
-      my $js = $self->_javascript_code($c, $config);
-      $$output =~ s!</head>!$js</head>!;
-    }
-  );
-}
-
-sub _default_route {
-  my ($self, $app, $config) = @_;
-
-  $config->{path} = '/mojolicious/plugin/surveil';
+  $app->hook(after_render => sub { _hook_after_render($config, @_) });
 
   $app->routes->websocket($config->{path})->to(
     cb => sub {
-      my $c = shift;
-      $c->inactivity_timeout(60);
-      $c->on(
-        message => sub {
-          my $action = decode_json $_[1];
-          my ($type, $target) = (delete $action->{type}, delete $action->{target});
-          $app->log->debug(qq(Event "$type" on "$target" @{[encode_json $action]}));
-        }
-      );
+      my $c = shift->inactivity_timeout(60);
+      $c->on(json => $config->{handler});
     }
   );
 }
 
-sub _javascript_code {
-  my ($self, $c, $config) = @_;
-  my $scheme = $c->req->url->to_abs->scheme || 'http';
+sub _default_message_handler {
+  my ($c,    $e)      = @_;
+  my ($type, $target) = delete @$e{qw(type target)};
+  $c->app->log->debug(qq(Event "$type" on "$target" @{[encode_json $e]}));
+}
 
+sub _hook_after_render {
+  my ($config, $c, $output, $format) = @_;
+  return if $format ne 'html';
+  return if !$c->param($config->{enable_param});
+
+  my $scheme = $c->req->url->to_abs->scheme || 'http';
   $scheme =~ s!^http!ws!;
 
-  $c->render_to_string(
+  my $js = $c->render_to_string(
     template    => 'mojolicious/plugin/surveil',
     events      => encode_json($config->{events}),
     surveil_url => $c->url_for($config->{path})->to_abs->scheme($scheme),
   );
+
+  $$output =~ s!</head>!$js</head>!;
 }
 
 1;
@@ -73,7 +57,7 @@ Mojolicious::Plugin::Surveil - Surveil user actions
 
 =head1 VERSION
 
-0.02
+0.03
 
 =head1 DESCRIPTION
 
@@ -81,8 +65,6 @@ L<Mojolicious::Plugin::Surveil> is a plugin which allow you to see every
 event a user trigger on your web page. It is meant as a debug tool for
 seeing events, even if the browser does not have a JavaScript console.
 
-Note: With great power, comes great responsibility.
-
 CAVEAT: The JavaScript that is injected require WebSocket in the browser to
 run. The surveil events are attached to the "body" element, so any other event
 that prevent events from bubbling will not emit this to the WebSocket
@@ -90,70 +72,80 @@ resource.
 
 =head1 SYNOPSIS
 
-Use default logging:
+=head2 Application
 
   use Mojolicious::Lite;
   plugin "surveil";
-  app->start;
 
-Use custom route:
+=head2 In your browser
 
-  use Mojolicious::Lite;
+Visit L<http://localhost:3000?_surveil=1> to enable the logging. Try clicking
+around on your page and look in the console for log messages.
 
-  plugin surveil => { path => "/surveil" };
+=head2 Custom event handler
 
-  websocket "/surveil" => sub {
-    my $c = shift;
+  use Mojo::Redis;
+  use Mojo::JSON "encode_json";
 
-    $c->on(message => sub {
-      my ($c, $action) = @_;
-      warn "User event: $action\n";
-    });
+  plugin "surveil", {
+    handler => sub {
+      my ($c, $event) = @_;
+      my $ip = $c->tx->remote_address;
+      $c->redis->pubsub->notify("surveil:$ip" => encode_json $event);
+    }
   };
 
-  app->start;
+The above example is useful if you want to publish the events to
+L<Redis|Mojo::Redis> instead of a log file. A developer can then run commands
+below to see what a given user is doing:
 
-=head1 CONFIG
-
-This plugin can take the following config params:
-
-=over 4
-
-=item * enable_param = "..."
-
-Used to specify a query parameter to be part of the URL to enable surveil.
-
-Default is not to require any query parameter.
-
-=item * events = [...]
-
-The events that should be reported back over the WebSocket.
-
-Defaults to click, touchstart, touchcancel and touchend.
-(The default list is EXPERIMENTAL).
-
-=item * path = "...";
-
-The path to the WebSocket route.
-
-Defaults to C</mojolicious/plugin/surveil>. Emitting the "path" parameter will
-also add a default WebSocket route which simply log with "debug" the action
-that was taken. (The format of the logging is EXPERIMENTAL)
-
-=back
+  $ redis-cli psubscribe "surveil:*"
+  $ redis-cli subscribe "surveil:192.168.0.100"
 
 =head1 METHODS
 
 =head2 register
 
-  $self->register($app, $config);
+  $self->register($app, \%config);
+  $app->plugin("surveil" => \%config);
 
 Used to add an "after_render" hook into the application which adds a
-JavaScript to every HTML document.
+JavaScript to every HTML document when the L</enable_param> is set.
+
+C<%config> can have the following settings:
+
+=over 2
+
+=item * enable_param
+
+Used to specify a query parameter to be part of the URL to enable surveil.
+
+Default is "_surveil".
+
+=item * events
+
+The events that should be reported back over the WebSocket.
+
+Defaults to blur, click, focus, touchstart, touchcancel and touchend.
+
+Note that the default list might change in the future.
+
+=item * handler
+
+A code ref that handles the events from the web page. This is useful if you
+want to post them to an event bus instead of in the log file.
+
+=item * path
+
+The path to the WebSocket route.
+
+Defaults to C</mojolicious/plugin/surveil>.
+
+=back
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2014, Jan Henning Thorsen
+Copyright (C) 2014-2018, Jan Henning Thorsen
 
 This program is free software, you can redistribute it and/or modify it under
 the terms of the Artistic License version 2.0.
@@ -167,10 +159,10 @@ Jan Henning Thorsen - C<jhthorsen@cpan.org>
 __DATA__
 @@ mojolicious/plugin/surveil.html.ep
 <script type="text/javascript">
-window.addEventListener("load", function(e) {
+(function(w) {
   var events = <%== $events %>;
   var socket = new WebSocket("<%= $surveil_url %>");
-  var console = window.console = window.console || {};
+  var console = w.console;
 
   console.surveil = function() {
     socket.send(JSON.stringify({type: "console", target: "window", message: Array.prototype.slice.call(arguments)}));
@@ -180,7 +172,7 @@ window.addEventListener("load", function(e) {
     socket.send(JSON.stringify({type: "load", target: "window"}));
     for (i = 0; i < events.length; i++) {
       document.body.addEventListener(events[i], function(e) {
-        var data = { extra: {} };
+        var data = {extra: {}};
         for (var prop in e) {
           if (!(typeof e[prop]).match(/^(boolean|number|string)$/)) continue;
           if (prop.match(/^[A-Z]/)) continue;
@@ -192,5 +184,5 @@ window.addEventListener("load", function(e) {
       });
     }
   }
-});
+})(window);
 </script>
